@@ -4,7 +4,7 @@ Text::CSV::Track::Min - same as Text::CSV::Track but stores the smallest value
 
 =head1 VERSION
 
-This documentation refers to version 0.2. 
+This documentation refers to version 0.3. 
 
 =head1 SYNOPSIS
 
@@ -19,7 +19,7 @@ old one. It it's higher then the value is updated if not old value persists.
 
 package Text::CSV::Track::Min;
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 use 5.006;
 
 use strict;
@@ -40,7 +40,7 @@ sub value_of {
 	my $value         = shift;
 
 	#variables from self hash
-	my $rh_value_of    = $self->{rh_value_of};
+	my $rh_value_of    = $self->_rh_value_of;
 
 	#check if we have identificator
 	return if not $identificator;
@@ -49,7 +49,10 @@ sub value_of {
 	return $self->SUPER::value_of($identificator) if not $is_set;
 	
 	#set
-	my $old_value = $rh_value_of->{$identificator};	#don't call SUPER::value_of because it will active lazy init that is may be not necessary
+	my $old_value;
+	if (exists $rh_value_of->{$identificator}) {	#don't call SUPER::value_of because it will active lazy init that is may be not necessary
+		$old_value = ${$rh_value_of->{$identificator}}[0];
+	}
 	if (not defined $value
 			or not defined $old_value
 			or ($old_value > $value)
@@ -58,3 +61,6 @@ sub value_of {
 		$self->SUPER::value_of($identificator, $value);
 	}
 }
+
+1;
+
